searchData={"items":[{"type":"module","title":"EctoCommand","doc":"The `EctoCommand` module provides a DSL for defining command schemas.\nIt is used by `use EctoCommand` in your command modules.","ref":"EctoCommand.html"},{"type":"module","title":"Example - EctoCommand","doc":"defmodule MyApp.Commands.CreatePost do\n      use EctoCommand\n      alias MyApp.PostRepository\n\n      command do\n        param :title, :string, required: true, length: [min: 3, max: 255]\n        param :body, :string, required: true, length: [min: 3]\n\n        internal :slug, :string\n        internal :author, :string\n      end\n\n      def execute(%__MODULE__{} = command) do\n        ...\n        :ok\n      end\n\n      def fill(:slug, _changeset, %{\"title\" => title}, _metadata) do\n        Slug.slufigy(title)\n      end\n\n      def fill(:author, _changeset, _params, %{\"triggered_by\" => triggered_by}) do\n        triggered_by\n      end\n\n      def fill(:author, changeset, _params, _metadata) do\n        Ecto.Changeset.add_error(changeset, :triggered_by, \"triggered_by metadata info is missing\")\n      end\n    end","ref":"EctoCommand.html#module-example"},{"type":"function","title":"EctoCommand.cast_embedded_fields/2","doc":"","ref":"EctoCommand.html#cast_embedded_fields/2"},{"type":"macro","title":"EctoCommand.command/1","doc":"The command/1 macro defines a command schema with the given block.","ref":"EctoCommand.html#command/1"},{"type":"macro","title":"Examples - EctoCommand.command/1","doc":"command do\n       param :name, :string\n       param :age, :integer\n     end","ref":"EctoCommand.html#command/1-examples"},{"type":"macro","title":"EctoCommand.embeds_one/3","doc":"","ref":"EctoCommand.html#embeds_one/3"},{"type":"macro","title":"EctoCommand.internal/3","doc":"Defines a command internal field.  \n\nThese fields will be ignored during the \"cast process\".\nInstead, you need to define a public `fill/4` function to populate them. The `fill/4` function takes four arguments: the name of the field, the current temporary changeset, the parameters received from external sources, and additional metadata. You can choose to return the value that will populate the field or the updated changeset. Both options are acceptable, but returning the changeset is particularly useful if you want to add errors to it.","ref":"EctoCommand.html#internal/3"},{"type":"macro","title":"Examples - EctoCommand.internal/3","doc":"internal :slug, :string\n     internal :author, :string","ref":"EctoCommand.html#internal/3-examples"},{"type":"macro","title":"EctoCommand.param/3","doc":"Defines a command parameter field.  \n\nThe `param` macro is based on the `field` macro of [Ecto.Schema](https://hexdocs.pm/ecto/Ecto.Schema.html) and defines a field in the schema with a given name and type. You can pass all the options supported by the `field` macro. Afterwards, each defined `param` is cast with the \"external\" data received.\nIn addition to those options, the `param` macro accepts a set of other options that indicate how external data for that field should be validated.\nThese options are applied to the intermediate Changeset created in order to validate data.\nThese options are mapped into `validate_*` methods of the [Ecto.Changeset](https://hexdocs.pm/ecto/Ecto.Changeset.html).","ref":"EctoCommand.html#param/3"},{"type":"macro","title":"Examples - EctoCommand.param/3","doc":"param :title, :string, required: true, length: [min: 3, max: 255]\n    param :body, :string, required: true, length: [min: 3]","ref":"EctoCommand.html#param/3-examples"},{"type":"macro","title":"EctoCommand.validate_with/2","doc":"","ref":"EctoCommand.html#validate_with/2"},{"type":"behaviour","title":"EctoCommand.Middleware","doc":"Middleware provides an extension point to add functions that you want to be\ncalled for every command execution\n\nImplement the `EctoCommand.Middleware` behaviour in your module and define the\n`c:before_execution/2`, `c:after_execution/2`, `c:after_failure/2` and `c:invalid/2` callback functions.","ref":"EctoCommand.Middleware.html"},{"type":"behaviour","title":"Example middleware - EctoCommand.Middleware","doc":"defmodule SampleMiddleware do\n      @behaviour EctoCommand.Middleware\n\n      @impl true\n      def before_execution(pipeline, _opts) do\n        pipeline\n        |> Pipeline.assign(:some_data, :some_value)\n        |> Pipeline.update!(:command, fn command -> %{command | name: \"updated-name\"} end)\n      end\n\n      @impl true\n      def after_execution(pipeline, _opts) do\n        Logger.debug(\"Command executed successfully\", command: pipeline.command, result: Pipeline.response(pipeline))\n\n        pipeline\n      end\n\n      @impl true\n      def after_failure(pipeline, _opts) do\n        Logger.error(\"Command execution fails\", command: pipeline.command, error: Pipeline.response(pipeline))\n\n        pipeline\n      end\n\n      @impl true\n      def invalid(pipeline, _opts) do\n        Logger.error(\"invalid params received\", params: pipeline.params, error: Pipeline.response(pipeline))\n\n        pipeline\n      end\n    end","ref":"EctoCommand.Middleware.html#module-example-middleware"},{"type":"callback","title":"EctoCommand.Middleware.after_execution/2","doc":"Is executed following a sucessful command execution. In this function, if you'd like, you could alter the returned value.","ref":"EctoCommand.Middleware.html#c:after_execution/2"},{"type":"callback","title":"Example - EctoCommand.Middleware.after_execution/2","doc":"@impl true\n    def after_execution(pipeline, _opts) do\n      Logger.debug(\"Command executed successfully\", command: pipeline.command, result: Pipeline.response(pipeline))\n\n      pipeline\n    end","ref":"EctoCommand.Middleware.html#c:after_execution/2-example"},{"type":"callback","title":"EctoCommand.Middleware.after_failure/2","doc":"Is executed after a failed command execution. In this function you could, if you wish, also update the returned value.","ref":"EctoCommand.Middleware.html#c:after_failure/2"},{"type":"callback","title":"Example - EctoCommand.Middleware.after_failure/2","doc":"@impl true\n    def after_failure(pipeline, _opts) do\n      Logger.error(\"Command execution fails\", command: pipeline.command, error: Pipeline.response(pipeline))\n\n      pipeline\n    end","ref":"EctoCommand.Middleware.html#c:after_failure/2-example"},{"type":"callback","title":"EctoCommand.Middleware.before_execution/2","doc":"Is executed before command execution, and only if the command is valid. In this function, if you'd like, you might update the command that will be executed.","ref":"EctoCommand.Middleware.html#c:before_execution/2"},{"type":"callback","title":"Example - EctoCommand.Middleware.before_execution/2","doc":"@impl true\n    def before_execution(pipeline, _opts) do\n      pipeline\n      |> Pipeline.assign(:some_data, :some_value)\n      |> Pipeline.update!(:command, fn command -> %{command | name: \"updated-name\"} end)\n    end","ref":"EctoCommand.Middleware.html#c:before_execution/2-example"},{"type":"callback","title":"EctoCommand.Middleware.invalid/2","doc":"Is executed when the command's inputs are invalid.","ref":"EctoCommand.Middleware.html#c:invalid/2"},{"type":"callback","title":"Example - EctoCommand.Middleware.invalid/2","doc":"@impl true\n    def invalid(pipeline, _opts) do\n      Logger.error(\"invalid params received\", params: pipeline.params, error: Pipeline.response(pipeline))\n\n      pipeline\n    end","ref":"EctoCommand.Middleware.html#c:invalid/2-example"},{"type":"type","title":"EctoCommand.Middleware.pipeline/0","doc":"","ref":"EctoCommand.Middleware.html#t:pipeline/0"},{"type":"module","title":"EctoCommand.Middleware.Pipeline","doc":"Pipeline is a struct used as an argument in the callback functions of modules\nimplementing the `EctoCommand.Middleware` behaviour.\n\nThis struct must be returned by each function to be used in the next\nmiddleware based on the configured middleware chain.","ref":"EctoCommand.Middleware.Pipeline.html"},{"type":"module","title":"Pipeline fields - EctoCommand.Middleware.Pipeline","doc":"- `assigns` - shared user data as a map.\n\n  - `command_uuid` - UUID assigned to the command being executed.\n\n  - `command` - command struct being executed.\n\n  - `params` - raw params received to instantiate the command\n\n  - `metadata` - additional metadata, they could be used to fill internal command fields\n\n  - `halted` - flag indicating whether the pipeline was halted.\n\n  - `handler` - handler module where the \"execute/1\" function resides\n\n  - `middlewares` - the list of middlewares to be executed\n\n  - `response` - sets the response to send back to the caller.\n\n  - `error` - sets the error to send back to the caller.","ref":"EctoCommand.Middleware.Pipeline.html#module-pipeline-fields"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.assign/3","doc":"Puts the `key` with value equal to `value` into `assigns` map.","ref":"EctoCommand.Middleware.Pipeline.html#assign/3"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.assign/3","doc":"iex> pipeline = assign(%Pipeline{}, :foo, :bar)\n    iex> pipeline.assigns\n    %{foo: :bar}","ref":"EctoCommand.Middleware.Pipeline.html#assign/3-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.chain/3","doc":"Executes the middleware chain.","ref":"EctoCommand.Middleware.Pipeline.html#chain/3"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.error/2","doc":"Sets the error","ref":"EctoCommand.Middleware.Pipeline.html#error/2"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.error/2","doc":"iex> pipeline = %Pipeline{}\n    iex> pipeline = Pipeline.error(pipeline, \"an_error\")\n    iex> Pipeline.response(pipeline)\n    {:error, \"an_error\"}","ref":"EctoCommand.Middleware.Pipeline.html#error/2-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.execute/1","doc":"Executes the function 'execute/1' in the handler module, pass the command to it.\nHalt the pipeline if command or handler are not set","ref":"EctoCommand.Middleware.Pipeline.html#execute/1"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.execute/1","doc":"iex> %Pipeline{halted: true} = Pipeline.execute(%Pipeline{halted: true})\n    iex> %Pipeline{response: {:error, \"command was not initialized\"}} = Pipeline.execute(%Pipeline{handler: Pipeline})\n    iex> %Pipeline{response: {:error, \"handler was not set\"}} = Pipeline.execute(%Pipeline{command: %{}})\n    iex> %Pipeline{response: {:ok, :result}} = Pipeline.execute(%Pipeline{handler: SampleCommand, command: %SampleCommand{}})","ref":"EctoCommand.Middleware.Pipeline.html#execute/1-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.halt/1","doc":"Halts the pipeline by preventing further middleware downstream from being invoked.\n\nPrevents execution of the command if `halt` occurs in a `before_execution` callback.","ref":"EctoCommand.Middleware.Pipeline.html#halt/1"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.halt/1","doc":"iex> pipeline = %Pipeline{}\n    iex> pipeline = halt(pipeline)\n    iex> halted?(pipeline)\n    true","ref":"EctoCommand.Middleware.Pipeline.html#halt/1-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.halt/2","doc":"Halts the pipeline by preventing further middleware downstream from being invoked.\n\nPrevents execution of the command if `halt` occurs in a `before_execution` callback.\n\nSimilar to `halt/1` but allows a response to be returned to the caller.","ref":"EctoCommand.Middleware.Pipeline.html#halt/2"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.halt/2","doc":"iex> pipeline = %Pipeline{}\n    iex> pipeline = halt(pipeline, {:error, \"halted\"})\n    iex> response(pipeline)\n    {:error, \"halted\"}\n    iex> halted?(pipeline)\n    true","ref":"EctoCommand.Middleware.Pipeline.html#halt/2-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.halted?/1","doc":"Has the pipeline been halted?","ref":"EctoCommand.Middleware.Pipeline.html#halted?/1"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.halted?/1","doc":"iex> true = halted?(%Pipeline{halted: true})\n    iex> false = halted?(%Pipeline{halted: false})","ref":"EctoCommand.Middleware.Pipeline.html#halted?/1-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.respond/2","doc":"Sets the response to be returned to the dispatch caller","ref":"EctoCommand.Middleware.Pipeline.html#respond/2"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.respond/2","doc":"iex> pipeline = %Pipeline{}\n    iex> pipeline = Pipeline.respond(pipeline, {:error, \"halted\"})\n    iex> Pipeline.response(pipeline)\n    {:error, \"halted\"}","ref":"EctoCommand.Middleware.Pipeline.html#respond/2-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.response/1","doc":"Extract the response from the pipeline, return the error if it is set\nreturn the stored response otherwise\nreturn nil if no response is set","ref":"EctoCommand.Middleware.Pipeline.html#response/1"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.response/1","doc":"iex> pipeline = %Pipeline{}\n    iex> pipeline = Pipeline.error(pipeline, \"halted\")\n    iex> Pipeline.response(pipeline)\n    {:error, \"halted\"}","ref":"EctoCommand.Middleware.Pipeline.html#response/1-examples"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.set/3","doc":"Set the `key` with value","ref":"EctoCommand.Middleware.Pipeline.html#set/3"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.set/3","doc":"iex> pipeline = set(%Pipeline{}, :command, :my_command)\n    iex> pipeline.command\n    :my_command","ref":"EctoCommand.Middleware.Pipeline.html#set/3-examples"},{"type":"type","title":"EctoCommand.Middleware.Pipeline.t/0","doc":"","ref":"EctoCommand.Middleware.Pipeline.html#t:t/0"},{"type":"function","title":"EctoCommand.Middleware.Pipeline.update!/3","doc":"Update the `key` with function `function` that receive the `key` value.","ref":"EctoCommand.Middleware.Pipeline.html#update!/3"},{"type":"function","title":"Examples - EctoCommand.Middleware.Pipeline.update!/3","doc":"iex> pipeline = %Pipeline{command: %{name: \"original\"}}\n    iex> pipeline = update!(pipeline, :command, fn command -> %{command | name: \"updated\"} end)\n    iex> pipeline.command\n    %{name: \"updated\"}","ref":"EctoCommand.Middleware.Pipeline.html#update!/3-examples"},{"type":"extras","title":"EctoCommand","doc":"# EctoCommand\n\nEctoCommand is a toolkit for mapping, validating, and executing commands received from any source.\nIt provides a simple and flexible way to define and execute commands in Elixir. With support for validation, middleware, and automatic OpenAPI documentation generation, it's a valuable tool for building scalable and maintainable Elixir applications. We hope you find it useful!","ref":"readme.html"},{"type":"extras","title":"Installation - EctoCommand","doc":"To install EctoCommand, add it as a dependency to your project by adding `ecto_command` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ecto_command, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Why Ecto? - EctoCommand","doc":"\"Ecto is also commonly used to map data from any source into Elixir structs, whether they are backed by a database or not.\"  \nBased on this definition of the [Ecto](https://github.com/elixir-ecto/ecto) library, **EctoCommand** utilizes the \"embedded_schema\" functionality to map input data into an Elixir data structure to be used as a \"command\".  \nThis means that **EctoCommand is not tied to your persistence layer**.\n\nAs a result, you can easily convert data received from any source into a valid command struct, which can be executed easily. Additionally, you can also add functionality through middlewares to the execution pipeline.\n\nHere is an example of a command definition:\n\n```elixir\ndefmodule SampleCommand do\n  use EctoCommand\n\n  command do\n    param :id, :string\n    param :name, :string, required: true, length: [min: 2, max: 255]\n    param :email, :string, required: true, format: ~r/@/, length: [min: 6]\n    param :count, :integer, required: true, number: [greater_than_or_equal_to: 18, less_than: 100]\n    param :password, :string, required: true, length: [greater_than_or_equal_to: 8, less_than: 100], trim: true\n\n    internal :hashed_password, :string\n  end\n\n  def execute(%SampleCommand{} = command) do\n    # ....\n    :ok\n  end\n\n  def fill(:hashed_password, _changeset, %{\"password\" => password}, _metadata) do\n    :crypto.hash(:sha256, password) |> Base.encode64()\n  end\nend\n\n:ok = SampleCommand.execute(%{id: \"aa-bb-cc\", name: \"foobar\", email: \"foo@bar.com\", count: 22, password: \"mysecret\"})\n\n```","ref":"readme.html#why-ecto"},{"type":"extras","title":"Usage - EctoCommand","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Defining a Command - EctoCommand","doc":"To define a new command, create a module that includes the `EctoCommand` behaviour and implements the `execute/1` function.  \nThe `execute/1` function takes the command structure as an argument.  \nThe `command` macro is used to define the parameters included in the command.   \nThe `param` macro is used to [define which parameters are accepted by the command](#params-definition), and the `internal` macro is used to [define which parameters are internally set](#internal-fields).\n\n\n```elixir\ndefmodule MyApp.Commands.CreatePost do\n  use EctoCommand\n\n  alias MyApp.PostRepository\n\n  command do\n    param :title, :string, required: true, length: [min: 3, max: 255]\n    param :body, :string, required: true, length: [min: 3]\n\n    internal :slug, :string\n    internal :author, :string\n  end\n\n  def execute(%__MODULE__{} = command) do\n    PostRepository.insert(%{\n      title: command.title,\n      body: command.body,\n      slug: command.slug\n    })\n  end\n\n  def fill(:slug, _changeset, %{\"title\" => title}, _metadata) do\n    Slug.slufigy(title)\n  end\n\n  def fill(:author, _changeset, _params, %{\"triggered_by\" => triggered_by}) do\n    triggered_by\n  end\n\n  def fill(:author, changeset, _params, _metadata) do\n    Ecto.Changeset.add_error(changeset, :triggered_by, \"triggered_by metadata info is missing\")\n  end\nend\n```","ref":"readme.html#defining-a-command"},{"type":"extras","title":"Executing a Command - EctoCommand","doc":"In order to execute the command, you need to call the `execute/2` function providing a raw parameter data map and, optionally, some metadata.  \n```elixir\nparams = %{title: \"New amazing post\", body: \"Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut eget ante odio.\"}\nmetadata = %{triggered_by: \"writer\"}\n\n:ok = MyApp.Commands.CreatePost.execute(params, metadata)\n```\nThis data is validated, and if it passes all validation rules, a new command structure is created and passed as an argument to the `execute/1` function defined inside your command module.","ref":"readme.html#executing-a-command"},{"type":"extras","title":"Handling Errors - EctoCommand","doc":"If a required parameter is missing or has an invalid value, the `EctoCommand.execute/2` function will return an error tuple with an invalid `Ecto.Changeset` structure. You can then use the changeset to return errors to the client or perform other actions.\n\n```elixir\n{:error, %Ecto.Changeset{valid?: false}} = MyApp.Commands.CreatePost.execute.execute(%{})\n```\n\nReturning an invalid `Ecto.Changeset` is particularly useful when working with Phoenix forms.","ref":"readme.html#handling-errors"},{"type":"extras","title":"Main goals and functionality of EctoCommand - EctoCommand","doc":"EctoCommand aims to provide the following functionality:\n\n- [An easy way to define the fields in a command (`param` macro).](#params-definition)\n- [A simple and compact way of specifying how these fields should be validated (`param` macro options)](#validations-and-constraints-definition)\n- [Defining the fields that need to be part of the command but can't be set from the outside (`internal` macro)](#internal-fields)\n- [Validation of the params received from the outside](#validations)\n- [Easy hooking of middleware to add functionality (like audit)](#using-middlewares-in-ectocommand)\n- [Automatic generation of OpenApi documentation](#automated-generation-of-openapi-documentation)","ref":"readme.html#main-goals-and-functionality-of-ectocommand"},{"type":"extras","title":"Params definition - EctoCommand","doc":"To define the params that a command should accept, use the `param` macro. \nThe `param` macro is based on the `field` macro of [Ecto.Schema](https://hexdocs.pm/ecto/Ecto.Schema.html) and defines a field in the schema with a given name and type. You can pass all the options supported by the `field` macro. Afterwards, each defined `param` is cast with the \"external\" data received.","ref":"readme.html#params-definition"},{"type":"extras","title":"Validations and constraints definition - EctoCommand","doc":"In addition to those options, the `param` macro accepts a set of other options that indicate how external data for that field should be validated. \nThese options are applied to the intermediate Changeset created in order to validate data. \nThese options are mapped into `validate_*` methods of the [Ecto.Changeset](https://hexdocs.pm/ecto/Ecto.Changeset.html). \nFor example, if you want a command to have a \"name\" field that is required and has a length between 2 and 255 chars, you can write:\n\n```Elixir\nparam :name, :string, required: true, length: [min: 2, max: 255]\n```\n\nThis means that the command will have a `name` field, which will be cast to a `string` type. These functions will be called during the changeset validation:\n```Elixir\nchangeset\n|> validate_required([:name])\n|> validate_length(:name, min: 2, max: 255)\n```\n\nfor validators that accept both data and options, you could pass just data like:\n```elixir\nparam :email, :string, format: ~r/@/\n```\nor data and options in this way:\n```elixir\nparam :email, :string, format: {~r/@/, message: \"my custom error message\"}\n```","ref":"readme.html#validations-and-constraints-definition"},{"type":"extras","title":"Internal fields - EctoCommand","doc":"Sometimes, you might need to define internal fields, like `hashed_password` or `triggered_by`, which are not supposed to be set externally. To define such fields, you can use the `internal` macro.\n\n```elixir\n  command do\n    param :password, :string, required: true, length: [greater_than_or_equal_to: 8, less_than: 100], trim: true\n    internal :hased_password, :string\n    internal :triggered_by, :string\n  end\n\n  def fill(:hased_password, _changeset, %{\"password\" => password}, _metadata) do\n    :crypto.hash(:sha256, password) |> Base.encode64()\n  end\n\n  def fill(:triggered_by, _changeset, _params, %{\"triggered_by\": triggered_by}) do\n    triggered_by\n  end\n\n  def fill(:triggered_by, changeset, _params, _metadata) do\n    Ecto.Changeset.add_error(changeset, :triggered_by, \"triggered_by metadata info is missing\")\n  end\n```\n\nThese fields will be ignored during the \"cast process\". Instead, you need to define a public `fill/4` function to populate them. The `fill/4` function takes four arguments: the name of the field, the current temporary changeset, the parameters received from external sources, and additional metadata. You can choose to return the value that will populate the field or the updated changeset. Both options are acceptable, but returning the changeset is particularly useful if you want to add errors to it.","ref":"readme.html#internal-fields"},{"type":"extras","title":"Subparams - EctoCommand","doc":"Your API can accept structured data by utilizing \"subparams\" within the request.  \nTo enable the submission of complex information in a hierarchical format, you can utilize the `embeds_one/3` macro. This macro allows you to define a field that is composed of other fields, which are subsequently validated.  \nThe arguments for the macro are: the name of the main parameter, the name of the embedded module (which can either exist or be created), and optionally, the parameters of the new embedded module.\n\nIn the following example, we demonstrate how you can provide a name, surname, and an address field, which is composed of street, city, and zip_code.\n```elixir\ndefmodule SampleCommand do\n  use EctoCommand\n\n  command do\n    param :name, :string\n    param :surname, :string\n\n    embeds_one :address, Address do\n      param :street, :string, required: true\n      param :city, :string, required: true, length: [min: 10]\n      param :zip, :string, required: true\n    end\n  end\nend\n```\nIn the above example, a new `SampleCommand.Address` module will be created.  \nAlternatively, you can achieve the same behavior by writing:\n```elixir\ndefmodule SampleCommand.Address do\n  use EctoCommand\n\n  command do\n    param :street, :string, required: true\n    param :city, :string, required: true, length: [min: 10]\n    param :zip, :string, required: true\n  end\nend\n\ndefmodule SampleCommand do\n  use EctoCommand\n\n  command do\n    param :name, :string\n    param :surname, :string\n\n    embeds_one :address, Address\n  end\nend\n```\nIn the second example, an existing module is embedded, providing the same functionality. Feel free to choose the approach that best suits your needs.","ref":"readme.html#subparams"},{"type":"extras","title":"Validations - EctoCommand","doc":"All parameters are validated in order to instantiate the command structure. \nWhen you use `EctoCommand` inside your module, three methods are added:\n\n- `changeset/2`\n- `new/2`\n- `execute/2`\n\nAll three methods take parameter data and metadata as arguments. \nThe `changeset/2` function performs validation and other operations, and returns a valid or invalid `Ecto.Changeset`.  \nThe `new/2` function internally calls the `changeset/2` function and returns either the valid command structure or the invalid `Ecto.Changeset`.  \nThe `execute/2` function internally calls the `new/2` function and then calls the `execute/1` function (which should be defined inside the command module), or returns the invalid `Ecto.Changeset`.","ref":"readme.html#validations"},{"type":"extras","title":"Using Middlewares in EctoCommand - EctoCommand","doc":"EctoCommand supports middlewares, which allow you to modify the behavior of a command before and/or after its execution. \n\nA middleware is a module that implements the `EctoCommand.Middleware` behavior. Here's how you can use middlewares in your EctoCommand project:\n\n```elixir\ndefmodule MyApp.MyMiddleware do\n  @behaviour EctoCommand.Middleware\n\n  @impl true\n  def before_execution(pipeline, _opts) do\n    pipeline\n    |> Pipeline.assign(:some_data, :some_value)\n    |> Pipeline.update!(:command, fn command -> %{command | name: \"updated-name\"} end)\n  end\n\n  @impl true\n  def after_execution(pipeline, _opts) do\n    Logger.debug(\"Command executed successfully\", command: pipeline.command, result: Pipeline.response(pipeline))\n    pipeline\n  end\n\n  @impl true\n  def after_failure(pipeline, _opts) do\n    Logger.error(\"Command execution fails\", command: pipeline.command, error: Pipeline.response(pipeline))\n    pipeline\n  end\n\n  @impl true\n  def invalid(pipeline, _opts) do\n    Logger.error(\"invalid params received\", params: pipeline.params, error: Pipeline.response(pipeline))\n    pipeline\n  end\nend\n```\n\nEach method takes two arguments: an [EctoCommand.Pipeline](https://github.com/silvadanilo/ecto_command/blob/master/lib/ecto_command/middleware/pipeline.ex) structure and the options you set for that middleware.  \nThe method should return an EctoCommand.Pipeline structure.\n\n- `before_execution/2` is executed before command execution, and only if the command is valid. In this function, if you'd like, you might update the command that will be executed.\n- `after_execution/2` is executed following a sucessful command execution. In this function, if you'd like, you could alter the returned value.\n- `after_failure/2` is executed after a failed command execution. In this function you could, if you wish, also update the returned value.\n- `invalid/2` is executed when data used to build the command is invalid.","ref":"readme.html#using-middlewares-in-ectocommand"},{"type":"extras","title":"Configuring Middlewares - EctoCommand","doc":"There are two ways to specify which middleware should be executed:\n\n1. **Global configuration:** \n\nYou can set up a list of middleware to be executed for every command by adding the following to your application's configuration:\n\n```elixir\nconfig :ecto_command, :middlewares,\n  {MyApp.MyFirstMiddleware, a_middleware_option: :foo},\n  {MyApp.MySecondMiddleware, a_middleware_option: :bar}\n```\n\n2. **Command-level configuration:** \n\nYou can also specify middleware to be executed for a specific command by adding the `use` directive in the command module:\n\n```elixir\ndefmodule MyApp.MyCommand do\n  use EctoCommand\n  use MyApp.MyFirstMiddleware, a_middleware_option: :foo\n  use MyApp.MySecondMiddleware, a_middleware_option: :bar\n\n  ....\nend\n```\n\nIn this case, the specified middleware is executed only for that particular command.","ref":"readme.html#configuring-middlewares"},{"type":"extras","title":"Automated generation of OpenAPI documentation - EctoCommand","doc":"EctoCommand has a built-in feature that automatically generates OpenAPI documentation based on the parameters and validation rules defined in your command modules.  \nThis can save you a significant amount of time and effort in writing and maintaining documentation, particularly if you have a large number of commands.  \n\nTo generate the OpenAPI schema, you can use the `EctoCommand.OpenApi` module:\n\n```elixir\nuse EctoCommand.OpenApi\n```\n\nBy default, the schema's title is the fully qualified domain name (FQDN) of the module, and the default type is `:object`. \nHowever, you can override the defaults by passing options to the `use` module:\n\n```elixir\nuse EctoCommand.OpenApi, title: \"CustomTitle\", type: :object\n```\n\n\nThen in your controller you can simply pass your command module to the request body specs:\n```elixir\n\ndefmodule MyAppWeb.PostController do\n  use MyAppWeb, :controller\n  use OpenApiSpex.ControllerSpecs\n\n  alias MyApp.Commands.CreatePost\n\n  operation :create_post,\n    summary: \"Create a Post\",\n    request_body: {\"Post params\", \"application/json\", CreatePost},\n    responses: [\n      ok: {\"Post response\", \"application/json\", []},\n      bad_request: {\"Post response\", \"application/json\", []}\n    ]\n  def create_post(conn, params) do\n    ...\n  end\n```\n\nFor more information on serving the Swagger UI, please refer to the readme of the [open-api-spex](https://github.com/open-api-spex/open_api_spex) library.","ref":"readme.html#automated-generation-of-openapi-documentation"},{"type":"extras","title":"Contributing - EctoCommand","doc":"Contributions are always welcome! Please feel free to submit a pull request or create an issue if you find a bug or have a feature request.","ref":"readme.html#contributing"},{"type":"extras","title":"License - EctoCommand","doc":"This library is licensed under the MIT license. See [LICENSE](https://github.com/silvadanilo/ecto_command/blob/master/LICENSE) for more details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}